declare function isEmail(str: string): boolean;

declare function replaceAt(str: string, index?: number, replaceStr?: string): string;

declare function insertAt(str: string, index?: number, insertStr?: string): string;

declare function capitalize(str: string): string;

declare function camelCase(str: string): string;

declare function pascalCase(str: string): string;

declare function arrayDiff(arr1: unknown[], arr2: unknown[]): unknown[];

/**
 * It generates a sequence of numbers starting at the first argument,
 * progressing by the third argument, and stopping at the second argument.
 *
 * @example
 *
 * range(0, 5) // [0, 1, 2, 3, 4, 5]
 * range(10, 50, 10) // [10, 20, 30, 40, 50]
 */
declare function range(start: number | undefined, stop: number, step?: number): number[];

/**
 *  Array filter with `Async` callback function.
 *
 * @param arr unknown[]
 * @param cb - The callback funtion
 * @returns The filtered array
 */
declare function asyncFilter(arr: unknown[], cb: (value: unknown, index: number) => Promise<boolean>): Promise<unknown[]>;

type GroupKey<T> = ((a: T) => string) | string;
declare function groupBy<T>(arr: T[], key: GroupKey<T>): Record<string, T[]>;

/**
 * It moves an array element from one index position to another.
 *
 * Note: It mutates the array.
 *
 * @example
 *
 * move([1, 2, 3], 0, 2) //=> [2, 3, 1]
 */
declare function move<T>(arr: T[], from: number, to: number): T[];

/**
 * It returns percentage of value & total.
 * @param {number} value - The current value
 * @param {number} total - The total value
 * @returns {number}
 */
declare function percentage(value: number, total: number): number;

/**
 * It returns percentage of a number.
 * @param {number} percentage - The percentage to calculate
 * @param {number} num - The number to find percentage
 * @returns {number}
 */
declare function percentageOf(percentage: number, num: number): number;

/**
 * Checks if the given value is number.
 *
 * @example
 *
 * isNum(1) //=> true
 *
 * isNum('1') //=> false
 *
 * isNum('1', true) //=> true
 *
 * isNum(NaN) //=> false
 *
 * isNum(Infinity) //=> false
 *
 */
declare function isNum(val: unknown, coerce?: boolean): boolean;

/**
 * Checks if the given value is a plain object.
 *
 * @example
 *
 * isObj({}) //=> true
 *
 * isObj([]) //=> false
 *
 * isObj(Object.create(null)) //=> true
 *
 */
declare function isObj(obj: unknown): boolean;

/**
 * Checks whether the given string is a valid JSON plain object
 *
 * @example
 *
 * isJSON("null") //=> false
 *
 * isJSON("[]") //=> false
 *
 * isJSON("{}") //=> true
 */
declare function isJSON(str: string): boolean;

/**
 * Checks if the given value is boolean.
 *
 * @example
 *
 * isBool(true) //=> true
 *
 * isBool(false) //=> true
 *
 * isBool(null) //=> false
 */
declare function isBool(val: unknown): boolean;

/**
 * Checks if the given value is an array.
 *
 * @example
 *
 * isArr([]) //=> true
 *
 * isArr(new Array('a', 'b', 'c')) //=> true
 *
 * isArr('abc') //=> false
 */
declare function isArr(val: unknown): boolean;

/**
 * Checks if the given value is a valid date object.
 *
 * @example
 *
 * isDate(new Date()) //=> true
 *
 * isDate(new Date('')) //=> false
 *
 * isDate('2000-01-01') //=> false
 */
declare function isDate(val: unknown): boolean;

/**
 * Checks if the given value is an error.
 *
 * @example
 *
 * isErr(new Error()) //=> true
 *
 * isErr({msg: '', name: ''}) //=> false
 */
declare function isErr(val: unknown): boolean;

/**
 * Checks if the given value is a function.
 *
 * @example
 *
 * isFn(function() {}) //=> true
 *
 * isFn({}) //=> false
 */
declare function isFn(val: unknown): boolean;

/**
 * Checks if the given value is a Map object.
 *
 * @example
 *
 * isMap(new Map()) //=> true
 *
 * isMap(new WeakMap()) //=> false
 *
 * isMap({}) //=> false
 *
 */
declare function isMap(val: unknown): boolean;

/**
 * Checks if the given value is null.
 *
 * @example
 *
 * isNull(null) //=> true
 *
 * isNull(undefined) //=> false
 *
 */
declare function isNull(val: unknown): boolean;

/**
 * Checks if the given value is a RegExp object.
 *
 * @example
 *
 * isRegEx(new RegExp()) //=> true
 *
 * isRegEx(/a/) //=> true
 *
 * isRegEx({}) //=> false
 *
 */
declare function isRegEx(obj: unknown): boolean;

/**
 * Checks if the given value is a Set object.
 *
 * @example
 *
 * isSet(new Set()) //=> true
 *
 * isSet(new WeakSet()) //=> false
 *
 * isSet({}) //=> false
 *
 */
declare function isSet(val: unknown): boolean;

/**
 * Checks if the given value is a string.
 *
 * @example
 *
 * isStr('abc') //=> true
 *
 * isStr(String(1)) //=> true
 *
 * isStr(1) //=> false
 *
 */
declare function isStr(val: unknown): boolean;

/**
 * Checks if the given value is a Symbol.
 *
 * @example
 *
 * isSym(Symbol()) //=> true
 *
 * isSym(Symbol('abc')) //=> true
 *
 * isSym('abc') //=> false
 *
 */
declare function isSym(val: unknown): boolean;

/**
 * Checks if the given value is a TypedArray.
 *
 * @example
 *
 * isTypedArr([]) //=> false
 *
 * isTypedArr(new Int8Array()) //=> true
 *
 * isTypedArr(new Float64Array()) //=> true
 */
declare function isTypedArr(val: unknown): boolean;

/**
 * Checks if the given value is an ArrayBuffer.
 *
 * @example
 *
 * isArrBuf([]) //=> false
 *
 * isArrBuf(new ArrayBuffer(8)) //=> true
 */
declare function isArrBuf(val: unknown): boolean;

/**
 * Checks if the given value is undefined.
 *
 * @example
 *
 * isUndef() //=> true
 *
 * isUndef(undefined) //=> true
 *
 * isUndef(null) //=> false
 *
 */
declare function isUndef(val: unknown): boolean;

/**
 * Checks if the given value is a WeakMap object.
 *
 * @example
 *
 * isWkMap(new Map()) //=> false
 *
 * isWkMap(new WeakMap()) //=> true
 *
 */
declare function isWkMap(val: unknown): boolean;

/**
 * Checks if the given value is a WeakSet object.
 *
 * @example
 *
 * isWkSet(new Set()) //=> false
 *
 * isWkSet(new WeakSet()) //=> true
 *
 */
declare function isWkSet(val: unknown): boolean;

/**
 * Checks if the given value is a WeakRef object.
 *
 * @example
 *
 * isWkRef({}) //=> false
 *
 * let user = { name: "John" };
 * isWkRef(new WeakRef(user)) //=> true
 *
 */
declare function isWkRef(val: unknown): boolean;

/**
 * Checks if the given value is a BigInt.
 *
 * @example
 *
 * isBigInt(1) //=> false
 *
 * isBigInt(1n) //=> true
 *
 */
declare function isBigInt(val: unknown): boolean;

/**
 * Checks if the given value is a blob.
 *
 * @example
 *
 * isBlob(new Blob()) //=> true
 *
 * isBlob({}) //=> false
 */
declare function isBlob(val: unknown): boolean;

declare function getInObj(obj: object, path: string): unknown;

declare function setInObj(obj: object, path: string, value: unknown): object;

declare function delInObj(obj: unknown, path: string): boolean;

declare function cloneObj(obj: any): any;

/**
 * Checks if two values are the same by shallow comparison.
 *
 * @example
 *
 * const a = {a: 1}
 * const b = {a: 1}
 *
 * a === b //=> false
 *
 * isShallowEql(a, b) //=> true
 *
 * isShallowEql([1, b], [1, b]) //=> true
 *
 * isShallowEql([1, {a: 1}], [1, {a: 1}]) //=> false
 */
declare function isShallowEql(val1: unknown, val2: unknown): boolean;

/**
 * Convert hexadecimal color values into RGB color values.
 * @param hex string
 * @returns number[]
 *
 * @example hexToRGB('#FF851B') => [ 255, 133, 27 ]
 */
declare function hexToRGB6(hex: string): number[];

declare function sleep(ms: number): Promise<void>;

export { arrayDiff, asyncFilter, camelCase, capitalize, cloneObj, delInObj, getInObj, groupBy, hexToRGB6 as hexToRGB, insertAt, isArr, isArrBuf, isBigInt, isBlob, isBool, isDate, isEmail, isErr, isFn, isJSON, isMap, isNull, isNum, isObj, isRegEx, isSet, isShallowEql, isStr, isSym, isTypedArr, isUndef, isWkMap, isWkRef, isWkSet, move, pascalCase, percentage, percentageOf, range, replaceAt, setInObj, sleep };
